#include "fs/devpts/Inode.h"

namespace DevPTS
{
    Result<SharedPtr<VFS::Inode>> RootInode::find(const char* name) const
    {
        for (const auto& entry : m_entries)
        {
            if (!strcmp(name, entry.name.chars())) return entry.inode;
        }

        return err(ENOENT);
    }

    Result<void> RootInode::replace_entry(SharedPtr<VFS::Inode> inode, const char* name)
    {
        for (auto& entry : m_entries)
        {
            if (!strcmp(name, entry.name.chars()))
            {
                entry.inode = inode;
                return {};
            }
        }

        return err(ENOENT);
    }

    Option<VFS::DirectoryEntry> RootInode::get(usize index) const
    {
        if (index >= m_entries.size()) return {};

        return m_entries[index];
    }

    Result<void> RootInode::add_entry(SharedPtr<VFS::Inode> inode, const char* name)
    {
        if (find(name).has_value()) return err(EEXIST);

        VFS::DirectoryEntry entry { inode, name };

        TRY(m_entries.try_append(move(entry)));

        inode->did_link();

        g_realtime_clock.get_time(m_metadata.mtime);

        return {};
    }

    Result<void> RootInode::remove_entry(const char* name)
    {
        SharedPtr<VFS::Inode> inode = TRY(find(name));

        if (inode->type() == VFS::InodeType::Directory && inode->entries() != 2) return err(ENOTEMPTY);

        if (inode->is_mountpoint()) return err(EBUSY);

        m_entries.remove_first_matching(
            [&](const VFS::DirectoryEntry& entry) { return !strcmp(entry.name.chars(), name); });

        inode->did_unlink();

        g_realtime_clock.get_time(m_metadata.mtime);

        return {};
    }

    Result<SharedPtr<VFS::Inode>> RootInode::create_file(const char*, mode_t)
    {
        return err(ENOTSUP);
    }

    Result<SharedPtr<VFS::Inode>> RootInode::create_subdirectory(const char*, mode_t)
    {
        return err(ENOTSUP);
    }
}
