#define _TIMESPEC_MACROS_INCLUDED
#include "Pledge.h"
#include "memory/MemoryManager.h"
#include "sys/Syscall.h"
#include "thread/Scheduler.h"
#include "thread/Timer.h"
#include <bits/itimer.h>
#include <luna/Common.h>

Result<u64> sys_setitimer(Registers*, SyscallArgs args)
{
    int which = (int)args[0];
    const struct itimerval* new_timer = (const struct itimerval*)args[1];
    struct itimerval* old_timer = (struct itimerval*)args[2];

    auto* current = Scheduler::current();

    TRY(check_pledge(current, Promise::p_stdio));

    Timer* timer;
    Clock* clock;
    switch (which)
    {
    case ITIMER_REAL:
        timer = &current->real_timer;
        clock = &g_realtime_clock;
        break;
    case ITIMER_VIRTUAL:
        timer = &current->virtual_timer;
        clock = &current->virtual_clock;
        break;
    case ITIMER_PROF:
        timer = &current->profiling_timer;
        clock = &current->profiling_clock;
        break;
    default: return err(EINVAL);
    }

    if (old_timer)
    {
        if (timer->clock)
        {
            struct itimerval result;
            auto total = timer->clock->from_ticks(timer->total_ticks);
            auto left = timer->clock->from_ticks(timer->clock->ticks_left(timer));
            result.it_interval = TIMESPEC_TO_TIMEVAL(total);
            result.it_value = TIMESPEC_TO_TIMEVAL(left);
            if (!MemoryManager::copy_to_user_typed(old_timer, &result)) return err(EFAULT);
        }
        else
        {
            struct itimerval result;
            memset(&result, 0, sizeof(result));
            if (!MemoryManager::copy_to_user_typed(old_timer, &result)) return err(EFAULT);
        }
    }

    if (new_timer)
    {
        if (timer->clock) timer->clock->remove_from_timer_queue(timer);

        struct itimerval itimer;
        if (!MemoryManager::copy_from_user_typed(new_timer, &itimer)) return err(EFAULT);

        timer->signo = SIGALRM; // FIXME: Also use SIGVTALRM or SIGPROF for other timer types.
        timer->thread = current;

        if (itimer.it_interval.tv_sec != 0 || itimer.it_interval.tv_usec != 0)
        {
            timer->restart = true;
            timer->total_ticks =
                (itimer.it_interval.tv_sec * 1'000'000'000 + itimer.it_interval.tv_usec * 1000) / clock->resolution();
        }
        else
            timer->restart = false;

        if (itimer.it_value.tv_sec != 0 || itimer.it_value.tv_usec != 0)
        {
            timer->delta_ticks =
                (itimer.it_value.tv_sec * 1'000'000'000 + itimer.it_value.tv_usec * 1000) / clock->resolution();
            clock->add_to_timer_queue(timer);
        }
    }

    return 0;
}
